@@ -1,4 +1,4 @@
-/*  $Id: syspred.pl,v 1.27 1996/10/30 17:31:34 jan Exp $
+/*  $Id: syspred.pl,v 1.30 1997/02/10 12:56:23 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     jan@swi.psy.uva.nl
@@ -56,6 +56,7 @@
 	, absolute_file_name/2
 	, absolute_file_name/3
 	, require/1
+	, call_with_depth_limit/3
 	]).	
 
 		/********************************
@@ -270,8 +271,8 @@ term_to_atom(Term, Atom) :-
 	$term_to_atom(Term, Atom, 0, 0).
 
 atom_to_term(Atom, Term, Bindings) :-
-	var(Bindings),
-	$term_to_atom(Term, Atom, Bindings, 0).
+	$term_to_atom(Term, Atom, B0, 0),
+	Bindings = B0.
 
 int_to_atom(Int, Atom) :-
 	int_to_atom(Int, 10, Atom).
@@ -318,6 +319,26 @@ prolog_load_context(directory, D) :-
 prolog_load_context(term_position, '$stream_position'(0,L,0,0,0)) :-
 	source_location(_, L).
 
+		 /*******************************
+		 *	      CONTROL		*
+		 *******************************/
+
+%	call_with_depth_limit(+Goal, +DepthLimit, -Result)
+%
+%	Try to proof Goal, but fail on any branch exceeding the indicated
+%	depth-limit.  Unify Result with the maximum-reached limit on success,
+%	depth_limit_exceeded if the limit was exceeded and fails otherwise.
+
+:- module_transparent call_with_depth_limit/3.
+
+call_with_depth_limit(G, Limit, Result) :-
+	$depth_limit(Limit, OLimit, OReached),
+	(   G,
+	    $depth_limit_true(Limit, OLimit, OReached, Result, Cut),
+	    Cut
+	;   $depth_limit_false(Limit, OLimit, OReached, Result)
+	).
+
 
 		/********************************
 		*           DATA BASE           *
@@ -424,6 +445,9 @@ $predicate_property(hashed(N), Pred) :-
 $predicate_property(references(N), Pred) :-
 	$get_predicate_attribute(Pred, references, N),
 	N > 0.
+$predicate_property(number_of_clauses(N), Pred) :-
+	$get_predicate_attribute(Pred, number_of_clauses, N),
+	N > 0.
 
 :- index(clause_property(0, 1)).
 
@@ -433,6 +457,8 @@ clause_property(Clause, file(File)) :-
 	$get_clause_attribute(Clause, file, File).
 clause_property(Clause, fact) :-
 	$get_clause_attribute(Clause, fact, true).
+clause_property(Clause, erased) :-
+	$get_clause_attribute(Clause, erased, true).
 
 
 :- module_transparent
