@@ -1,4 +1,4 @@
-/*  $Id: toplevel.pl,v 1.27 1996/10/15 08:17:14 jan Exp $
+/*  $Id: toplevel.pl,v 1.29 1997/03/10 09:36:09 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     jan@swi.psy.uva.nl
@@ -167,11 +167,13 @@ $search_aliases([Alias|More]) -->
 	$search_aliases(More).
 $search_aliases([Alias]) -->
 	$string(AliasChars),
-	[], !,
+	$eos, !,
 	{ $make_alias(AliasChars, Alias) }.
 
 $string(X) --> {X=[_|_]}, X.
 
+$eos([], []).
+
 $make_alias(Chars, Alias) :-
 	term_to_atom(Alias, Chars),
 	(   atom(Alias)
@@ -249,6 +251,8 @@ $break :-
 	$ttyformat('[exit break level ~d]~n', [New]),
 	flag($break_level, _, Old), !.
 
+:- $hide($toplevel, 0).			% avoid in the GUI stacktrace
+
 $toplevel :-
 	$runtoplevel,
 	$ttyformat('[halt]~n', []).		
@@ -265,14 +269,6 @@ $compile :-
 	$compile_wic.
 
 
-$calleventhook(Term) :-
-	(   notrace(user:prolog_event_hook(Term))
-	->  true
-	;   true
-	).
-
-:- $hide($calleventhook, 1).
-
 		/********************************
 		*    USER INTERACTIVE LOOP      *
 		*********************************/
@@ -376,13 +372,13 @@ $execute_goal(Goal, Bindings) :-
 	    (	$write_bindings(NewBindings)
 	    ->	!,
 	        notrace,
-		$calleventhook(finished_query(Qid)),
+		$calleventhook(finished_query(Qid, true)),
 		erase(Ref),
 		fail
 	    )
 	;   notrace, 
 	    $ttyformat('~nNo~n'),
-	    $calleventhook(finished_query(Qid)),
+	    $calleventhook(finished_query(Qid, false)),
 	    erase(Ref),
 	    fail
 	).
