@@ -1,4 +1,4 @@
-/*  $Id: pl-arith.c,v 1.29 1996/10/15 08:18:12 jan Exp $
+/*  $Id: pl-arith.c,v 1.31 1997/02/10 12:56:40 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -614,7 +614,7 @@ UNAIRY_FLOAT_FUNCTION(ar_exp, exp)
 UNAIRY_FLOAT_FUNCTION(ar_log10, log10)
 
 BINAIRY_FLOAT_FUNCTION(ar_atan2, atan2)
-BINAIRY_FLOAT_FUNCTION(ar_pow, pow);
+BINAIRY_FLOAT_FUNCTION(ar_pow, pow)
 
 BINAIRY_INT_FUNCTION(ar_mod, %)
 BINAIRY_INT_FUNCTION(ar_div, /)
@@ -627,9 +627,9 @@ BINAIRY_INT_FUNCTION(ar_xor, ^)
 static int
 ar_sign(Number n1, Number r)
 { if ( intNumber(n1) )
-    r->value.i = (n1->value.i < 0 ? -1 : n1->value.i > 0 ? 1 : 0);
+    r->value.i = (n1->value.i <   0 ? -1 : n1->value.i >   0 ? 1 : 0);
   else
-    r->value.f = (n1->value.f < 0 ? -1 : n1->value.f > 0 ? 1 : 0);
+    r->value.i = (n1->value.f < 0.0 ? -1 : n1->value.f > 0.0 ? 1 : 0);
 
   r->type = V_INTEGER;
   succeed;
