@@ -1,4 +1,4 @@
-/*  $Id: pl-ext.c,v 1.55 1996/10/30 17:32:35 jan Exp $
+/*  $Id: pl-ext.c,v 1.57 1997/03/10 09:36:30 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -115,13 +115,18 @@ static struct foreign {
   FRG("break",			0, pl_break,			TRACE_ME),
   FRG("$break",			1, pl_break1,			TRACE_ME),
   FRG("notrace",		1, pl_notrace1,			META),
+#ifdef O_LIMIT_DEPTH
+  FRG("$depth_limit",		3, pl_depth_limit,		TRACE_ME),
+  FRG("$depth_limit_true",	5, pl_depth_limit_true,	   NDET|TRACE_ME),
+  FRG("$depth_limit_false",	4, pl_depth_limit_false,	TRACE_ME),
+#endif
 
   FRG("display",		1, pl_display,			TRACE_ME),
   FRG("displayq",		1, pl_displayq,			TRACE_ME),
   FRG("write",			1, pl_write,			TRACE_ME),
   FRG("writeq",			1, pl_writeq,			TRACE_ME),
   FRG("print",			1, pl_print,			TRACE_ME),
-  FRG("$print",			2, pl_dprint,		META|TRACE_ME),
+  FRG("$print",			2, pl_dprint,		   META|TRACE_ME),
 
   FRG("read_variables",		2, pl_read_variables,		TRACE_ME),
   FRG("read_term",		2, pl_read_term,		TRACE_ME),
@@ -182,6 +187,7 @@ static struct foreign {
   FRG("nth_clause", 		3, pl_nth_clause,     NDET|META|TRACE_ME),
   FRG("retract",    		1, pl_retract,        NDET|META|TRACE_ME),
   FRG("retractall",		1, pl_retractall,	   META|TRACE_ME),
+#if O_DEBUGGER
   FRG("$xr_member",		2, pl_xr_member,      NDET|META|TRACE_ME),
   FRG("$wam_list",		1, pl_wam_list,			TRACE_ME),
   FRG("$fetch_vm",		4, pl_fetch_vm,			TRACE_ME),
@@ -189,7 +195,8 @@ static struct foreign {
   FRG("$break_pc",		3, pl_break_pc,		   NDET|TRACE_ME),
   FRG("$break_at",		3, pl_break_at,			TRACE_ME),
   FRG("$current_break",		2, pl_current_break,	   NDET|TRACE_ME),
-  
+#endif  
+
   FRG("flag",			3, pl_flag,			TRACE_ME),
   FRG("recorda",		3, pl_recorda,			TRACE_ME),
   FRG("recordz",		3, pl_recordz,			TRACE_ME),
