@@ -41,6 +41,9 @@ handling times must be cleaned, but that not only holds for this module.
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_BSTRING_H
+#include <bstring.h>
+#endif
 
 #define ST_TERMINAL 0			/* terminal based stream */
 #define ST_FILE	    1			/* File bound stream */
@@ -78,14 +81,14 @@ typedef struct input_context * InputContext;
 typedef struct output_context * OutputContext;
 
 static struct input_context
-{ int	stream;				/* pushed input */
-  atom_t	term_file;			/* old term_position file */
-  int	term_line;			/* old term_position line */
-  InputContext previous;		/* previous context */
+{ int		stream;			/* pushed input */
+  atom_t	term_file;		/* old term_position file */
+  int		term_line;		/* old term_position line */
+  InputContext	previous;		/* previous context */
 } *input_context_stack = NULL;
 
 static struct output_context
-{ int	stream;				/* pushed input */
+{ int		stream;			/* pushed input */
   OutputContext previous;		/* previous context */
 } *output_context_stack = NULL;
 
@@ -254,7 +257,7 @@ push/popInputContext() maintain the source_location   info  over see(X),
 context if it concerns a new stream and seen() will only pop if it is an
 open stream.
 
-Should be fixed decently if we redesign all of I/O stream manegement.
+Should be fixed decently if we redesign all of I/O stream management.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static void
@@ -329,8 +332,10 @@ getSingleChar(void)
 { int c;
   int OldIn = Input;
   ttybuf buf;
+  IOSTREAM *stream;
     
   Input = 0;
+  stream = fileTable[Input].stream;
   debugstatus.suspendTrace++;
   pl_ttyflush();
   PushTty(&buf, TTY_RAW);		/* just donot prompt */
@@ -345,7 +350,13 @@ getSingleChar(void)
     while( c2 != EOF && c2 != '\n' )	/* read upto newline */
       c2 = Get0();
   } else
-    c = Get0();
+  { if ( stream->position )
+    { IOPOS oldpos = *stream->position;
+      c = Get0();
+      *stream->position = oldpos;
+    } else
+      c = Get0();
+  }
 
   PopTty(&buf);
   debugstatus.suspendTrace--;
@@ -572,7 +583,7 @@ openStream(term_t file, int mode, int flags)
   } else
     return warning("Illegal stream specification");
 
-  DEBUG(3, Sdprintf("File/command name = %s\n", stringAtom(f)));
+  DEBUG(3, Sdprintf("File/command name = %s\n", name));
   if ( type == ST_FILE )
   { if ( mode == F_READ )
     { if ( name == ATOM_user || name == ATOM_user_input )
@@ -589,7 +600,7 @@ openStream(term_t file, int mode, int flags)
 	succeed;
       }
     }
-  } else if ( type == ST_PIPE && (mode == F_APPEND || mode == F_UPDATE) )
+  } else if ( type == ST_PIPE && (mode == F_APPEND || mode == F_WRNOTRUNC) )
     return warning("Cannot open a pipe in `append' or `update' mode");
     
   if ( !(flags & OPEN_OPEN) )		/* see/1, tell/1, append/1 */
@@ -599,7 +610,7 @@ openStream(term_t file, int mode, int flags)
 	{ switch(mode)
 	  { case F_READ:	Input = n; break;
 	    case F_WRITE:
-	    case F_UPDATE:
+	    case F_WRNOTRUNC:
 	    case F_APPEND:	Output = n; break;
 	  }
 	  DEBUG(3, Sdprintf("Switched back to already open stream %d\n", n));
@@ -660,7 +671,7 @@ openStream(term_t file, int mode, int flags)
   { case F_READ:
       Input = n; break;
     case F_WRITE:
-    case F_UPDATE:
+    case F_WRNOTRUNC:
     case F_APPEND:
       mode = F_WRITE;
       Output = n; break;
@@ -844,11 +855,15 @@ seenString()
 
 
 bool
-tellString(char *s, int size)
-{ IOSTREAM *stream = Sopen_string(NULL, s, size, "w");
+tellString(char **s, int size)
+{ static int sbuf;
+  IOSTREAM *stream;
   PlFile f;
   int n;
   
+  sbuf = size;
+  stream = Sopenmem(s, &sbuf, "w");
+ 
   for(n=3, f=&fileTable[n]; n<maxfiles; n++, f++)
   { if ( !f->stream )
     { f->stream = stream;
@@ -871,7 +886,8 @@ toldString()
 { PlFile f = &fileTable[Output];
 
   if ( f->type == ST_STRING && f->stream )
-  { Sclose(f->stream);
+  { Sputc(EOS, f->stream);
+    Sclose(f->stream);
     f->stream = NULL;
     f->status = F_CLOSED;
     popOutputContext();
@@ -882,7 +898,7 @@ toldString()
 
 
 		/********************************
-		*        INPUT IOSTREAM NAME        *
+		*        INPUT IOSTREAM NAME    *
 		*********************************/
 
 atom_t
@@ -1261,7 +1277,7 @@ pl_open4(term_t file, term_t mode,
     else if ( mname == ATOM_append )
       m = F_APPEND;
     else if ( mname == ATOM_update )
-      m = F_UPDATE;
+      m = F_WRNOTRUNC;
     else if ( mname == ATOM_read )
       m = F_READ;
   }
@@ -1281,6 +1297,7 @@ pl_open4(term_t file, term_t mode,
 	    s->flags |= SIO_FEOF2ERR;
 	}
 	Input = in;
+	pushInputContext();
         succeed;
       }
       closeStream(Input);
@@ -1411,7 +1428,7 @@ streamNo(term_t spec, int mode)
 	return warning("Stream is not open for reading");
       break;
     case F_APPEND:
-    case F_UPDATE:
+    case F_WRNOTRUNC:
     case F_WRITE:	
       if ( fileTable[n].status != F_WRITE )
       { warning("Stream is not open for writing");
@@ -1426,11 +1443,16 @@ streamNo(term_t spec, int mode)
 word
 pl_close(term_t stream)
 { int n;
+  int isread;
 
   if ( (n = streamNo(stream, F_ANY)) < 0 )
     fail;
+  isread = (fileTable[n].status == F_READ);
 
   TRY( closeStream(n) );
+  if ( isread )
+    popInputContext();
+  
   if ( n == Output )
     Output = 1;
   if ( n == Input )
