@@ -1,4 +1,4 @@
-/*  $Id: pl-funcs.h,v 1.71 1996/10/30 17:32:37 jan Exp $
+/*  $Id: pl-funcs.h,v 1.75 1997/03/10 09:36:34 jan Exp $
 
     Part of SWI-Prolog
     Designed and implemented by Jan Wielemaker
@@ -50,11 +50,7 @@ bool		unify_atomic(Word p, word a);
 word		pl_alt(term_t skip, word h);
 void		TrailAssignment(Word p);
 void		DoTrail(Word p);
-#ifdef __WIN32__
 void		do_undo(mark *m);
-#else
-inline void	do_undo(mark *m);
-#endif
 void		fix_term_ref_count(void);
 
 /* pl-atom.c */
@@ -185,7 +181,7 @@ word		pl_noprotocol(void);
 bool		seeString(char *s);
 bool		seeingString(void);
 bool		seenString(void);
-bool		tellString(char *s, int size);
+bool		tellString(char **s, int size);
 bool		toldString(void);
 atom_t		currentStreamName(void);
 word		pl_wait_for_input(term_t streams, term_t avail, term_t tmo);
@@ -475,6 +471,15 @@ word		pl_novice(term_t old, term_t new);
 word		pl_break(void);
 word		pl_break1(term_t goal);
 word		pl_notrace1(term_t goal);
+#ifdef O_LIMIT_DEPTH
+word		pl_depth_limit(term_t limit, term_t olimit, term_t oreached);
+word		pl_depth_limit_true(term_t limit,
+				    term_t olimit, term_t oreached,
+				    term_t res, term_t cut, word b);
+word		pl_depth_limit_false(term_t limit,
+				     term_t olimit, term_t oreached,
+				     term_t res);
+#endif /*O_LIMIT_DEPTH*/
 int		callProlog(Module module, term_t goal, int debug);
 word		pl_abort(void);
 bool		prolog(atom_t toplevel);
@@ -574,7 +579,7 @@ void		resetStacks(void);
 word		pl_trim_stacks(void);
 word		pl_limit_stack(term_t s, term_t l);
 word		pl_stack_parameter(term_t s, term_t k, term_t o, term_t n);
-void		mapOrOutOf(Stack s);
+void		ensureRoomStack(Stack s, int n);
 
 
 /* pl-sys.c */
