@@ -1,4 +1,4 @@
-/*  $Id: pl-gc.c,v 1.36 1996/10/30 17:32:41 jan Exp $
+/*  $Id: pl-gc.c,v 1.38 1997/03/10 09:36:39 jan Exp $
 
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
@@ -179,7 +179,7 @@ static long relocation_cells;	/* # relocation cells */
 static long relocated_cells;	/* # relocated cells */
 static long needs_relocation;	/* # cells that need relocation */
 static long local_marked;	/* # cells marked local -> global ptrs */
-#if O_SHIFT_STACKS || O_SECURE || defined(O_MAINTENANCE)
+#if O_SHIFT_STACKS || O_SECURE || defined(O_MAINTENANCE) || defined(O_DEBUG)
 static long local_frames;	/* frame count for debugging */
 #endif
 #if O_SECURE
@@ -378,7 +378,7 @@ forward:				/* Go into the tree */
       next--;				/* last cell of term */
       val = get_value(next);		/* invariant */
 					/* backwards pointer (NO ref!) */
-      set_value(next, consPtr(current, TAG_COMPOUND|STG_GLOBAL));
+      set_value(next, makePtr(current, TAG_COMPOUND));
       current = next;
       FORWARD;
     }
@@ -410,7 +410,7 @@ backward:  				/* reversing backwards */
     if ( isRef(w) )
       val = makeRef(current);
     else
-      val = consPtr(current-1, TAG_COMPOUND|STG_GLOBAL);
+      val = makePtr(current-1, TAG_COMPOUND);
     current= next;
   }
 
@@ -481,11 +481,16 @@ clearUninitialisedVarsFrame(LocalFrame fr, Code PC)
     for( ; ; PC += (codeTable[c].arguments + 1))
     { c = decode(*PC);
 
+#if O_DEBUGGER
     again:
+#endif
       switch( c )
-      { case D_BREAK:
+      {
+#if O_DEBUGGER
+	case D_BREAK:
 	  c = decode(replacedBreak(PC));
 	  goto again;
+#endif
 	case I_EXIT:
 	case I_EXITFACT:
 	  return;
