@@ -1,4 +1,4 @@
-/*  $Id: pl-incl.h,v 1.88 1996/10/30 17:32:44 jan Exp $
+/*  $Id: pl-incl.h,v 1.93 1997/03/10 09:36:43 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -17,7 +17,7 @@
 #define ARCH	     "i386-win32"
 #define C_LIBS	     "-lreadline -lconsole -luxnt"
 #define C_STATICLIBS ""
-#define C_CC	     "msvc++ 2.0"
+#define C_CC	     "msvc++ 4.2"
 #define C_LDFLAGS    ""
 #else
 #include <parms.h>			/* pick from the working dir */
@@ -87,6 +87,7 @@ handy for it someone wants to add a data type to the system.
 #define O_INTERRUPT		1
 #define O_DESTRUCTIVE_ASSIGNMENT 1
 #define O_HASHTERM		1
+#define O_LIMIT_DEPTH		1
 
 #ifndef DOUBLE_TO_LONG_CAST_RAISES_SIGFPE
 #ifdef __i386__
@@ -94,6 +95,16 @@ handy for it someone wants to add a data type to the system.
 #endif
 #endif
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+On the MIPS, whether or not alignment can   be used appears to depend on
+the compilation flags. We play safe for   the  case the user changes the
+flags after running configure.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#if !defined(DOUBLE_ALIGNMENT) && defined(__mips__)
+#define DOUBLE_ALIGNMENT sizeof(double)
+#endif
+
 #if MMAP_STACK || O_SHARED_MEMORY || HAVE_VIRTUAL_ALLOC
 #define O_DYNAMIC_STACKS 1		/* sparse memory management */
 #else
@@ -326,13 +337,15 @@ them.  Descriptions:
 #define OP_MAXPRIORITY		1200	/* maximum operator priority */
 #define SMALLSTACK		200 * 1024 /* GC policy */
 
+#define WORDBITSIZE		(8 * sizeof(word))
+
 				/* Prolog's integer range */
-#define PLMINTAGGEDINT		(-(1L<<(32 - LMASK_BITS - 1)))
+#define PLMINTAGGEDINT		(-(long)(1L<<(WORDBITSIZE - LMASK_BITS - 1)))
 #define PLMAXTAGGEDINT		(-PLMINTAGGEDINT - 1)
 #define inTaggedNumRange(n)	(((n)&~PLMAXTAGGEDINT) == 0 || \
 				 ((n)&~PLMAXTAGGEDINT) == ~PLMAXTAGGEDINT)
-#define PLMAXINT		((long)0x7fffffffL)
-#define PLMININT		((long)0x80000000L)
+#define PLMININT		((long)(1L<<(WORDBITSIZE-1)))
+#define PLMAXINT		(-(PLMININT+1))
 
 #if vax
 #define MAXREAL			(1.701411834604692293e+38)
@@ -383,7 +396,7 @@ sizes  of  the  hash  tables are defined.  Note that these should all be
 	  ForeignRedoPtr(v); \
 	}
 
-#define NEED_REINDEX 0x80000000L	/* defenition needs reindexing */
+#define NEED_REINDEX (1L << (8*sizeof(long)-1)) /* defenition reindexing */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Foreign language interface definitions.  Note that these macros MUST  be
@@ -434,7 +447,7 @@ codes.
 #define B_RFUNCTOR	((code) 6)		/* start functor */
 #define H_FUNCTOR	((code) 7)
 #define H_RFUNCTOR	((code) 8)
-#define I_POP		((code) 9)		/* end functor */
+#define I_POPF		((code) 9)		/* end functor */
 #define B_VAR		((code)10)		/* variable */
 #define H_VAR		((code)11)
 #define B_CONST		((code)12)		/* constant (atomic) */
@@ -563,7 +576,7 @@ Files and streams.  Don't change the numbers, or change FOPENMODE below.
 #define F_READ		1		/* open for reading */
 #define F_WRITE		2		/* open for writing */
 #define F_APPEND	3		/* open for append writing */
-#define F_UPDATE	4		/* open for write without truncate */
+#define F_WRNOTRUNC	4		/* open for write without truncate */
 #define F_ANY		5		/* any mode (finding stream) */
 
 #define FOPENMODE "-rwau"		/* Sopen_file() mode argument */
@@ -809,12 +822,13 @@ with one operation, it turns out to be faster as well.
 Handling environment (or local stack) frames.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define FR_LEVEL		(0xFFFFFF00L)
-#define FR_CUT			(0x00000001L)
-#define FR_NODEBUG		(0x00000002L)
-#define FR_SKIPPED		(0x00000004L)
-#define FR_MARKED		(0x00000008L)
-#define FR_CHOICEPT		(0x00000010L)
+#define FR_LEVEL		(~0xffL)
+#define FR_CUT			(0x01L)
+#define FR_NODEBUG		(0x02L)
+#define FR_SKIPPED		(0x04L)
+#define FR_MARKED		(0x08L)
+#define FR_CHOICEPT		(0x10L)
+#define FR_WATCHED		(0x20L)
 
 #define ARGOFFSET		((int) sizeof(struct localFrame))
 
@@ -836,6 +850,7 @@ Handling environment (or local stack) frames.
 #define contextModule(f)	((f)->context)
 
 #if O_DEBUG
+#define enterDefinition(def)	{ def->references++; }
 #define leaveDefinition(def)	{ if ( --def->references == 0 && \
 				       true(def, NEEDSCLAUSEGC) ) \
 				    gcClausesDefinition(def); \
@@ -844,23 +859,37 @@ Handling environment (or local stack) frames.
 					     predicateName(def)); \
 				    def->references = 0; \
 				  } \
+				  if ( true(fr, FR_WATCHED) ) \
+				    frameFinished(fr); \
 				}
 #else /*O_DEBUG*/
+#define enterDefinition(def)	{ def->references++; }
 #define leaveDefinition(def)	{ if ( --def->references == 0 && \
 				       true(def, NEEDSCLAUSEGC) ) \
 				    gcClausesDefinition(def); \
 				}
 #endif /*O_DEBUG*/
 
+#ifdef O_DEBUGGER
+#define leaveFrame(fr) { if ( true(fr->predicate, FOREIGN) ) \
+			 { if ( true(fr->predicate, NONDETERMINISTIC) ) \
+			     leaveForeignFrame(fr); \
+			 } else \
+			   leaveDefinition(fr->predicate); \
+			 if ( true(fr, FR_WATCHED) ) \
+			   frameFinished(fr); \
+		       }
+#else /*O_DEBUGGER*/
 #define leaveFrame(fr) { if ( true(fr->predicate, FOREIGN) ) \
 			 { if ( true(fr->predicate, NONDETERMINISTIC) ) \
 			     leaveForeignFrame(fr); \
 			 } else \
 			   leaveDefinition(fr->predicate); \
 		       }
+#endif /*O_DEBUGGER*/
 
-#define INVALID_TRAILTOP  0xffffffffL
-#define INVALID_GLOBALTOP 0xfffffffeL
+#define INVALID_TRAILTOP  ((word)~0L)
+#define INVALID_GLOBALTOP ((word)~1L)
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Heuristics functions to determine whether an integer reference passed to
@@ -915,7 +944,7 @@ struct index
 
 struct functorDef
 { FunctorDef	next;		/* next in chain */
-  word		functor;	/* as apearing on the global stack */
+  word		functor;	/* as appearing on the global stack */
   word		name;		/* Name of functor */
   int		arity;		/* arity of functor */
   unsigned short	flags;	/* Flag field holding: */
@@ -1055,6 +1084,12 @@ struct queryFrame
     LocalFrame  bfr;
   } registers;
 #endif
+#ifdef O_LIMIT_DEPTH
+  long		saved_depth_limit;	/* saved values of these */
+  long		saved_depth_reached;
+#endif
+  int		debug;			/* FALSE: nondebugging call */
+  int		debugSave;		/* saved debugstatus.debugging */
   Word	       *aSave;			/* saved argument-stack */
   int		solutions;		/* # of solutions produced */
   int		deterministic;		/* Last success was deterministic */
@@ -1218,12 +1253,13 @@ typedef struct
 		 *	      EVENTS		*
 		 *******************************/
 
-#define PLEV_ERASED	0		/* clause or record was erased */
-#define PLEV_DEBUGGING	1		/* changed debugging mode */
-#define PLEV_TRACING	2		/* changed tracing mode */
-#define PLEV_SPY	3		/* changed spypoint */
-#define PLEV_BREAK	4		/* a break-point was set */
-#define PLEV_NOBREAK	5		/* a break-point was cleared */
+#define PLEV_ERASED	   0 		/* clause or record was erased */
+#define PLEV_DEBUGGING	   1		/* changed debugging mode */
+#define PLEV_TRACING	   2		/* changed tracing mode */
+#define PLEV_SPY	   3		/* changed spypoint */
+#define PLEV_BREAK	   4		/* a break-point was set */
+#define PLEV_NOBREAK	   5		/* a break-point was cleared */
+#define PLEV_FRAMEFINISHED 6		/* A watched frame was discarded */
 
 
 		/********************************
@@ -1332,7 +1368,7 @@ GLOBAL char *	hBase;			/* lowest allocated heap address */
 #ifdef NO_SEGV_HANDLING
 #define requireStack(s, n) \
 	{ if ( roomStack(s) < (int)(n) ) \
- 	    mapOrOutOf((Stack)&stacks.s); \
+ 	    ensureRoomStack((Stack)&stacks.s, n); \
 	}
 #define verifyStack(s) requireStack(s, 64)
 #else /*NO_SEGV_HANDLING*/
@@ -1369,7 +1405,8 @@ GLOBAL long	  source_char_no;	/* Current source charno */
 GLOBAL bool	  fileerrors;		/* Report file errors? */
 GLOBAL atom_t	  float_format;		/* Default floating point format */
 
-#define ReadingSource (source_line_no > 0 && source_file_name != NULL_ATOM)
+#define ReadingSource (source_line_no > 0 && \
+		       source_file_name != NULL_ATOM)
 
 		/********************************
 		*        FAST DISPATCHING	*
@@ -1535,6 +1572,12 @@ GLOBAL struct _feature
 
 #define trueFeature(mask)	true(&features, mask)
 
+#ifdef O_LIMIT_DEPTH
+#define DEPTH_NO_LIMIT	(~0x0L)		/* Highest value */
+GLOBAL unsigned long depth_limit;	/* recursion depth limit */
+GLOBAL unsigned long depth_reached;	/* Deapest point reached */
+#endif
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Administration of loaded intermediate code files  (see  pl-wic.c).  Used
 with the -c option to include all these if necessary.
