@@ -1,4 +1,4 @@
-/*  $Id: pl-load.c,v 1.36 1996/07/19 14:07:42 jan Exp $
+/*  $Id: pl-load.c,v 1.37 1997/01/20 16:23:27 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -188,7 +188,7 @@ pl_load_foreign(term_t file, term_t entry, term_t options,
 		term_t libraries, term_t size)
 { char *sfile, *sentry, *soptions, *slibraries;
   long sz, nsz, n;
-  Atom execName;
+  atom_t execName;
   char *execFile;
   long base;
   int fd;
