@@ -1,4 +1,4 @@
-/*  $Id: pl-main.c,v 1.52 1996/11/11 16:27:26 jan Exp $
+/*  $Id: pl-main.c,v 1.53 1997/01/20 16:23:29 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -425,6 +425,10 @@ startProlog(int argc, char **argv, char **env)
 			} else
 			  usage();
 			break;
+	case 'p':	if (!argc)	/* handled in Prolog */
+			  usage();
+			argc--, argv++;
+			break;
 	case 'O':	status.optimise = TRUE;
 			break;
   	case 'o':	optionString(options.compileOut);
