@@ -18,6 +18,7 @@
 #include <os2.h>                /* this has to appear before pl-incl.h */
 #endif
 
+#include <math.h>			/* avoid abs() problem with msvc++ */
 #include "pl-incl.h"
 #include "pl-ctype.h"
 
@@ -387,7 +388,7 @@ Random(void)
   l ^= rand()<<10;
   l ^= rand()<<20;
 
-  return l & 0x7fffffff;
+  return l & (~PLMININT);
 #endif
 }
 
@@ -404,8 +405,8 @@ available to the Prolog user based on these functions.  These  functions
 are  in  this  module as non-UNIX OS probably don't have getpid() or put
 temporaries on /tmp.
 
-    Atom TemporaryFile(id)
-	 char *id;
+    atom_t TemporaryFile(id)
+	   char *id;
 
     The return value of this call is an atom,  whose  string  represents
     the  path  name of a unique file that can be used as temporary file.
@@ -1732,14 +1733,16 @@ ResetTty()
   prompt_next = TRUE;
 }
 
-#ifdef O_TERMIO				/* sys/termios.h or sys/termio.h */
+#ifdef O_HAVE_TERMIO			/* sys/termios.h or sys/termio.h */
 
+#ifndef HAVE_TCSETATTR
 #ifndef NO_SYS_IOCTL_H_WITH_SYS_TERMIOS_H
 #include <sys/ioctl.h>
 #endif
 #ifndef TIOCGETA
 #define TIOCGETA TCGETA
 #endif
+#endif
 
 bool
 PushTty(ttybuf *buf, int mode)
@@ -1751,14 +1754,26 @@ PushTty(ttybuf *buf, int mode)
   if ( status.notty )
     succeed;
 
+#ifdef HAVE_TCSETATTR 
+  if ( tcgetattr(0, &buf->tab) )	/* save the old one */
+    fail;
+#else
   if ( ioctl(0, TIOCGETA, &buf->tab) )	/* save the old one */
     fail;
+#endif
+
   tio = buf->tab;
 
   switch( mode )
   { case TTY_RAW:
+#if defined(HAVE_TCSETATTR) && defined(HAVE_CFMAKERAW)
+	cfmakeraw(&tio);
+	tio.c_oflag = buf->tab.c_oflag;	/* donot change output modes */
+	tio.c_lflag |= ISIG;
+#else
 	tio.c_lflag &= ~(ECHO|ICANON);
 	tio.c_cc[VTIME] = 0, tio.c_cc[VMIN] = 1;
+#endif
 	break;
     case TTY_OUTPUT:
 	tio.c_oflag |= (OPOST|ONLCR);
@@ -1770,11 +1785,20 @@ PushTty(ttybuf *buf, int mode)
 	/*NOTREACHED*/
   }
 
+#ifdef HAVE_TCSETATTR
+  if ( tcsetattr(0, TCSANOW, &tio) != 0 )
+  { static int warned;
+
+    if ( !warned++ )
+      warning("Failed to set terminal: %s", OsError());
+  }
+#else
 #ifdef TIOCSETAW
   ioctl(0, TIOCSETAW, &tio);
 #else
   ioctl(0, TCSETAW, &tio);
   ioctl(0, TCXONC, (void *)1);
+#endif
 #endif
 
   succeed;
@@ -1788,17 +1812,21 @@ PopTty(ttybuf *buf)
   if ( status.notty )
     succeed;
 
+#ifdef HAVE_TCSETATTR
+  tcsetattr(0, TCSANOW, &buf->tab);
+#else
 #ifdef TIOCSETA
   ioctl(0, TIOCSETA, &buf->tab);
 #else
   ioctl(0, TCSETA, &buf->tab);
   ioctl(0, TCXONC, (void *)1);
+#endif
 #endif
 
   succeed;
 }
 
-#else /* O_TERMIO */
+#else /* O_HAVE_TERMIO */
 
 #ifdef HAVE_SGTTYB
 
@@ -1876,7 +1904,7 @@ ttybuf *buf;
 }
 
 #endif /*HAVE_SGTTYB*/
-#endif /*O_TERMIO*/
+#endif /*O_HAVE_TERMIO*/
 
 
 		/********************************
