@@ -1,4 +1,4 @@
-/*  $Id: pl-os.h,v 1.32 1996/10/30 17:32:54 jan Exp $
+/*  $Id: pl-os.h,v 1.34 1997/03/10 09:36:54 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -182,25 +182,31 @@ extern real	  CpuTime(void);
 #define TTY_OUTPUT	 3		/* enable post-processing */
 #define TTY_SAVE	 4		/* just save status */
 
+#ifdef HAVE_TCSETATTR
+#include <termios.h>
+#include <unistd.h>
+#define O_HAVE_TERMIO 1
+#else /*HAVE_TCSETATTR*/
 #ifdef HAVE_SYS_TERMIO_H
 #include <sys/termio.h>
 #define termios termio
-#define O_TERMIO 1
+#define O_HAVE_TERMIO 1
 #else
 #ifdef HAVE_SYS_TERMIOS_H
 #include <sys/termios.h>
-#define O_TERMIO 1
+#define O_HAVE_TERMIO 1
 #endif
 #endif
+#endif /*HAVE_TCSETATTR*/
 
-#ifdef O_TERMIO
+#ifdef O_HAVE_TERMIO
 
 typedef struct
 { struct termios tab;		/* saved tty status */
   int		mode;		/* Prolog;'s view on mode */
 } ttybuf;
 
-#else /* !O_TERMIO */
+#else /* !O_HAVE_TERMIO */
 
 #ifdef HAVE_SGTTYB
 #include <sys/ioctl.h>
@@ -216,7 +222,7 @@ typedef struct
 } ttybuf;
 
 #endif /*HAVE_SGTTYB*/
-#endif /*O_TERMIO*/
+#endif /*O_HAVE_TERMIO*/
 
 extern ttybuf	ttytab;			/* saved tty status */
 extern int	ttymode;		/* Current tty mode */
