@@ -1,4 +1,4 @@
-/*  $Id: pl-pro.c,v 1.32 1996/10/30 17:32:58 jan Exp $
+/*  $Id: pl-pro.c,v 1.34 1997/02/10 12:56:59 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -84,24 +84,17 @@ pl_notrace1(term_t goal)
 
   long	     skipSave  = debugstatus.skiplevel;
   bool	     traceSave = debugstatus.tracing;
-  bool	     debugSave = debugstatus.debugging;
-  int	     suspSave  = debugstatus.suspendTrace;
-
-  debugstatus.tracing = FALSE;
-  debugstatus.debugging = FALSE;
-  debugstatus.suspendTrace = 1;
 
   rval = callProlog(NULL, goal, FALSE);
 
-  debugstatus.suspendTrace = suspSave;
   debugstatus.skiplevel    = skipSave;
-  debugstatus.debugging    = debugSave;
   debugstatus.tracing      = traceSave;
 
   return rval;
 }
 
 
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Call a prolog goal from C. The argument must  be  an  instantiated  term
 like for the Prolog predicate call/1.
@@ -200,6 +193,10 @@ prolog(volatile atom_t goal)
   gTop = gBase;
   aTop = aBase;
 
+#ifdef O_LIMIT_DEPTH
+  depth_limit   = (unsigned long)DEPTH_NO_LIMIT;
+#endif
+
   PL_open_foreign_frame();
 
   gc_status.blocked    = 0;
@@ -216,7 +213,7 @@ prolog(volatile atom_t goal)
   can_abort = TRUE;
   { fid_t cid = PL_open_foreign_frame();
     Procedure p = lookupProcedure(lookupFunctorDef(goal, 0), MODULE_system);
-    qid_t qid  = PL_open_query(MODULE_system, FALSE, p, 0);
+    qid_t qid  = PL_open_query(MODULE_system, TRUE, p, 0);
     rval = PL_next_solution(qid);
     PL_close_query(qid);
     PL_discard_foreign_frame(cid);
