@@ -1,4 +1,4 @@
-/*  $Id: pl-proc.c,v 1.47 1996/11/11 13:35:41 jan Exp $
+/*  $Id: pl-proc.c,v 1.49 1997/03/10 09:37:00 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -369,7 +369,7 @@ removeClausesProcedure(Procedure proc, int sfindex)
 { Definition def = proc->definition;
   ClauseRef c;
 
-  def->references++;
+  enterDefinition(def);
 
   for(c = def->definition.clauses; c; c = c->next)
   { Clause cl = c->clause;
@@ -432,9 +432,11 @@ retractClauseProcedure(Procedure proc, Clause clause)
 
 
   	freeClauseRef(c);
+#if O_DEBUGGER
 	if ( PROCEDURE_event_hook1 &&
 	     def != PROCEDURE_event_hook1->definition )
 	  callEventHook(PLEV_ERASED, clause);
+#endif
 	freeClause(clause);
 	def->number_of_clauses--;
 
@@ -452,8 +454,11 @@ retractClauseProcedure(Procedure proc, Clause clause)
 
 void
 freeClause(Clause c)
-{ if ( true(c, HAS_BREAKPOINTS) )
+{
+#if O_DEBUGGER
+  if ( true(c, HAS_BREAKPOINTS) )
     clearBreakPointsClause(c);
+#endif
 
   statistics.codes -= c->code_size;
   freeHeap(c->codes, sizeof(code) * c->code_size);
@@ -499,8 +504,10 @@ gcClausesDefinition(Definition def)
       }
 
       DEBUG(0, removed++);
+#if O_DEBUGGER
       if ( PROCEDURE_event_hook1 && def != PROCEDURE_event_hook1->definition )
 	callEventHook(PLEV_ERASED, c->clause);
+#endif
       freeClause(c->clause);
       freeClauseRef(c);
     } else
@@ -562,7 +569,7 @@ pl_check_definition(term_t spec)
 
   ClauseRef cref;
 
-  if ( get_procedure(spec, &proc, 0, GP_FIND) )
+  if ( !get_procedure(spec, &proc, 0, GP_FIND) )
     return warning("$check_definition/1: can't find definition");
   def = proc->definition;
 
@@ -709,14 +716,12 @@ trapUndefined(Definition def)
       PL_put_integer( argv+2, def->functor->arity);
       PL_put_variable(argv+3);
 
-      debugstatus.suspendTrace++;
       undefined_nesting++;
       qid = PL_open_query(MODULE_system, FALSE, pred, argv);
       if ( PL_next_solution(qid) )
 	PL_get_atom(argv+3, &answer);
       PL_close_query(qid);
       undefined_nesting--;
-      debugstatus.suspendTrace--;
       source_file_name = sfn;
       source_line_no   = sln;
       PL_discard_foreign_frame(cid);
@@ -799,7 +804,7 @@ pl_retract(term_t term, word h)
 		predicateName(def), def->references);
 
       cref = def->definition.clauses;
-      def->references++;			/* reference the predicate */
+      enterDefinition(def);			/* reference the predicate */
     } else
     { cref = ForeignContextPtr(h);
       proc = cref->clause->procedure;
@@ -863,7 +868,7 @@ pl_retractall(term_t head)
   if ( true(def, LOCKED) && false(def, DYNAMIC) )
     return warning("retractall/1: Attempt to retract from a system predicate");
 
-  def->references++;
+  enterDefinition(def);
   for(cref = def->definition.clauses; cref; cref = cref->next)
   { bool det;
     Word argv;
@@ -1010,7 +1015,10 @@ pl_get_predicate_attribute(term_t pred,
   } else if ( key == ATOM_references )
   { return PL_unify_integer(value, def->references);
   } else if ( key == ATOM_number_of_clauses )
-  { return PL_unify_integer(value, def->number_of_clauses);
+  { if ( def->flags & FOREIGN )
+      fail;
+
+    return PL_unify_integer(value, def->number_of_clauses);
   } else if ( (att = attribute_mask(key)) )
   { return PL_unify_integer(value, (def->flags & att) ? 1 : 0);
   } else
@@ -1191,6 +1199,9 @@ pl_get_clause_attribute(term_t ref, term_t att, term_t value)
   { return PL_unify_atom(value,
 			 true(clause, UNIT_CLAUSE) ? ATOM_true
 			 			   : ATOM_false);
+  } else if ( a == ATOM_erased )
+  { return PL_unify_atom(value,
+			 true(clause, ERASED) ? ATOM_true : ATOM_false);
   }
 
   fail;
