@@ -1,14 +1,12 @@
-/*  $Id: pl-read.c,v 1.40 1996/10/30 17:33:03 jan Exp $
+/*  $Id: pl-read.c,v 1.44 1997/03/10 09:37:03 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
     jan@swi.psy.uva.nl
-
-    Purpose: read/1, 2
 */
 
 #include <math.h>
-#define O_DEBUG 1
+/*#define O_DEBUG 1*/
 #include "pl-incl.h"
 #include "pl-ctype.h"
 
@@ -191,11 +189,7 @@ errorWarning(char *what)
       Sfprintf(Serror, "%s]\n", token_start);
     }
   } else
-  { fid_t cid = PL_open_foreign_frame();
-    qid_t qid;
-    term_t argv = PL_new_term_refs(3);
-    term_t a    = PL_new_term_ref();
-    predicate_t pred = PL_pred(FUNCTOR_exception3, MODULE_user);
+  { predicate_t pred = PL_pred(FUNCTOR_exception3, MODULE_user);
     int rval;
     char *s;
 
@@ -204,17 +198,25 @@ errorWarning(char *what)
       	source_line_no++;
     }
 	
-    PL_put_atom(    argv+0, ATOM_syntax_error);
-    PL_put_functor( argv+1, FUNCTOR_syntax_error3);
-    PL_put_variable(argv+2);
-    PL_get_arg(1, argv+1, a); PL_unify_atom(a, source_file_name);
-    PL_get_arg(2, argv+1, a); PL_unify_integer(a, source_line_no);
-    PL_get_arg(3, argv+1, a); PL_unify_atom_chars(a, what);
+    if ( pred->definition->definition.clauses )
+    { fid_t cid = PL_open_foreign_frame();
+      qid_t qid;
+      term_t argv = PL_new_term_refs(3);
+      term_t a    = PL_new_term_ref();
+    
+      PL_put_atom(    argv+0, ATOM_syntax_error);
+      PL_put_functor( argv+1, FUNCTOR_syntax_error3);
+      PL_put_variable(argv+2);
+      PL_get_arg(1, argv+1, a); PL_unify_atom(a, source_file_name);
+      PL_get_arg(2, argv+1, a); PL_unify_integer(a, source_line_no);
+      PL_get_arg(3, argv+1, a); PL_unify_atom_chars(a, what);
 	       
-    qid = PL_open_query(MODULE_user, FALSE, pred, argv);
-    rval = PL_next_solution(qid);
-    PL_close_query(qid);
-    PL_discard_foreign_frame(cid);
+      qid = PL_open_query(MODULE_user, FALSE, pred, argv);
+      rval = PL_next_solution(qid);
+      PL_close_query(qid);
+      PL_discard_foreign_frame(cid);
+    } else
+      rval = FALSE;
 
     if ( !rval )
       warning("%s:%d: Syntax error: %s",
@@ -362,7 +364,14 @@ raw_read2()
 		  return rb.base;
 		}
 		if (something_read)
-		{ rawSyntaxError("Unexpected end of file");
+		{ if ( dotseen )		/* term.<EOF> */
+		  { if ( rb.here - rb.base == 1 )
+		      rawSyntaxError("Unexpected end of clause");
+		    ensure_space(' ');
+		    addToBuffer(EOS);
+		    return rb.base;
+		  }
+		  rawSyntaxError("Unexpected end of file");
 		}
 		if ( Sfpasteof(ioi) )
 		{ warning("Attempt to read past end-of-file");
@@ -1218,10 +1227,6 @@ PL_assign_term(term_t to, term_t from)
 
 #define priorityClash { syntaxError("Operator priority clash"); }
 
-#ifndef MAX_TERM_NESTING
-#define MAX_TERM_NESTING 200
-#endif
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This part of the parser actually constructs  the  term.   It  calls  the
 tokeniser  to  find  the next token and assumes the tokeniser implements
@@ -1237,12 +1242,14 @@ typedef struct
   short	right_pri;			/* priority at right hand */
   short	op_pri;				/* priority of operator */
   term_t tpos;				/* term-position */
+  char *token_start;			/* start of the token for message */
 } op_entry;
 
 
 typedef struct
 { term_t term;				/* the term */
   term_t tpos;				/* its term-position */
+  int	 pri;				/* priority of the term */
 } out_entry;
 
 
@@ -1280,37 +1287,59 @@ build_op_term(term_t term, atom_t atom, int arity, out_entry *argv)
   build_term(term, atom, arity, av);
 }
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+realloca() maintains a buffer using  alloca()   that  has  the requested
+size. The current size is maintained in   a long just below the returned
+area. This is a long, to ensure   proper  allignment of the remainder of
+the values.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#define realloca(p, unit, n) \
+	{ long *ip = (long *)(p); \
+	  if ( ip == NULL || ip[-1] < (n) ) \
+	  { long nsize = (((n)+(n)/2) + 3) & ~3; \
+	    long *np = alloca(nsize * unit + sizeof(long)); \
+	    *np++ = nsize; \
+	    if ( ip ) \
+	      memcpy(np, ip, ip[-1] * unit); \
+	    p = (void *)np; \
+	  } \
+	}
+
 #define PushOp() \
+	realloca(side, sizeof(*side), side_n+1); \
 	side[side_n++] = in_op; \
-	if ( side_n >= MAX_TERM_NESTING ) \
-	  syntaxError("Operator stack overflow"); \
-	side_p = (side_n == 1 ? side : side_p+1);
+	side_p = (side_n == 1 ? 0 : side_p+1);
 	
-#define Modify(pri) \
-	if ( side_p != NULL && pri > side_p->right_pri ) \
+#define Modify(cpri) \
+	if ( side_p >= 0 && cpri > side[side_p].right_pri ) \
 	{ term_t tmp; \
-	  if ( side_p->kind == OP_PREFIX && rmo == 0 ) \
+	  if ( side[side_p].kind == OP_PREFIX && rmo == 0 ) \
 	  { DEBUG(1, Sdprintf("Prefix %s to atom\n", \
-			      stringAtom(side_p->op))); \
+			      stringAtom(side[side_p].op))); \
 	    rmo++; \
 	    tmp = PL_new_term_ref(); \
-	    PL_put_atom(tmp, side_p->op); \
+	    PL_put_atom(tmp, side[side_p].op); \
+	    realloca(out, sizeof(*out), out_n+1); \
 	    out[out_n].term = tmp; \
-	    out[out_n++].tpos = side_p->tpos; \
+	    out[out_n].tpos = side[side_p].tpos; \
+	    out[out_n].pri = 0; \
+	    out_n++; \
 	    side_n--; \
-	    side_p = (side_n == 0 ? NULL : side_p-1); \
-	  } else if ( side_p->kind == OP_INFIX && out_n > 0 && rmo == 0 && \
-		      isOp(side_p->op, OP_POSTFIX, side_p) ) \
+	    side_p = (side_n == 0 ? -1 : side_p-1); \
+	  } else if ( side[side_p].kind == OP_INFIX && out_n > 0 && rmo == 0 && \
+		      isOp(side[side_p].op, OP_POSTFIX, &side[side_p]) ) \
 	  { DEBUG(1, Sdprintf("Infix %s to postfix\n", \
-			      stringAtom(side_p->op))); \
+			      stringAtom(side[side_p].op))); \
 	    rmo++; \
 	    tmp = PL_new_term_ref(); \
-	    build_op_term(tmp, side_p->op, 1, &out[out_n-1]); \
+	    build_op_term(tmp, side[side_p].op, 1, &out[out_n-1]); \
+	    out[out_n-1].pri  = side[side_p].op_pri; \
 	    out[out_n-1].term = tmp; \
-	    side_p->kind = OP_POSTFIX; \
-	    out[out_n-1].tpos = opPos(side_p, &out[out_n-1]); \
+	    side[side_p].kind = OP_POSTFIX; \
+	    out[out_n-1].tpos = opPos(&side[side_p], &out[out_n-1]); \
 	    side_n--; \
-	    side_p = (side_n == 0 ? NULL : side_p-1); \
+	    side_p = (side_n == 0 ? -1 : side_p-1); \
 	  } \
 	}
 
@@ -1370,32 +1399,100 @@ opPos(op_entry *op, out_entry *args)
   return 0;
 }
 
-#define Reduce(cond) \
-	while( out_n > 0 && side_p != NULL && (cond) ) \
-	{ int arity = (side_p->kind == OP_INFIX ? 2 : 1); \
+
+static int
+can_reduce(out_entry *out, op_entry *op)
+{ int rval;
+
+  switch(op->kind)
+  { case OP_PREFIX:
+      rval = op->right_pri >= out[0].pri;
+      break;
+    case OP_POSTFIX:
+      rval = op->left_pri >= out[0].pri;
+      break;
+    case OP_INFIX:
+      rval = op->left_pri >= out[0].pri &&
+	     op->right_pri >= out[1].pri;
+      break;
+    default:
+      assert(0);
+      rval = FALSE;
+  }
+
+  return rval;
+}
+
+static int
+bad_operator(out_entry *out, op_entry *op)
+{ char buf[1024];
+  term_t t;
+  atom_t name;
+  int arity;
+  char *opname = stringAtom(op->op);
+
+  token_start = op->token_start;
+
+  switch(op->kind)
+  { case OP_INFIX:
+      if ( op->left_pri < out[0].pri )
+	t = out[0].term;
+      else
+      { token_start += strlen(opname);
+	t = out[1].term;
+      }
+      break;
+    case OP_PREFIX:
+      token_start += strlen(opname);
+      /*FALL THROUGH*/
+    default:
+      t = out[0].term;
+  }
+
+  if ( PL_get_name_arity(t, &name, &arity) )
+  { Ssprintf(buf, "Operator `%s' conflicts with `%s'",
+	     opname, stringAtom(name));
+  } else
+  { Ssprintf(buf, "Unknown conflict with operator `%s'", opname);
+  }
+
+  syntaxError(buf);
+
+  fail;
+}
+
+#define Reduce(cpri) \
+	while( out_n > 0 && side_p >= 0 && (cpri) >= side[side_p].op_pri ) \
+	{ int arity = (side[side_p].kind == OP_INFIX ? 2 : 1); \
 	  term_t tmp; \
- 	  if ( arity > out_n ) break; \
+	  if ( arity > out_n ) break; \
+	  if ( !can_reduce(&out[out_n-arity], &side[side_p]) ) \
+          { if ( (cpri) == (OP_MAXPRIORITY+1) ) \
+              return bad_operator(&out[out_n-arity], &side[side_p]); \
+	    break; \
+	  } \
 	  DEBUG(1, Sdprintf("Reducing %s/%d\n", \
-			    stringAtom(side_p->op), arity));\
+			    stringAtom(side[side_p].op), arity));\
 	  tmp = PL_new_term_ref(); \
 	  out_n -= arity; \
-	  build_op_term(tmp, side_p->op, arity, &out[out_n]); \
+	  build_op_term(tmp, side[side_p].op, arity, &out[out_n]); \
+	  out[out_n].pri  = side[side_p].op_pri; \
 	  out[out_n].term = tmp; \
-	  out[out_n].tpos = opPos(side_p, &out[out_n]); \
+	  out[out_n].tpos = opPos(&side[side_p], &out[out_n]); \
 	  out_n ++; \
 	  side_n--; \
-	  side_p = (side_n == 0 ? NULL : side_p-1); \
+	  side_p = (side_n == 0 ? -1 : side_p-1); \
 	}
 
 
 static bool
 complex_term(const char *stop, term_t term, term_t positions)
-{ out_entry out[MAX_TERM_NESTING];
-  op_entry  side[MAX_TERM_NESTING];
+{ out_entry *out  = NULL;
+  op_entry  *side = NULL;
   op_entry  in_op;
   int out_n = 0, side_n = 0;
   int rmo = 0;				/* Rands more than operators */
-  op_entry *side_p = NULL;
+  int side_p = -1;
   term_t pin;
 
   for(;;)
@@ -1433,6 +1530,7 @@ complex_term(const char *stop, term_t term, term_t positions)
 
       PL_get_atom(in, &name);
       in_op.tpos = pin;
+      in_op.token_start = token_start;
 
       DEBUG(1, Sdprintf("name %s, rmo = %d\n", stringAtom(name), rmo));
 
@@ -1441,7 +1539,7 @@ complex_term(const char *stop, term_t term, term_t positions)
 
 	Modify(in_op.left_pri);
 	if ( rmo == 1 )
-	{ Reduce(in_op.op_pri > side_p->right_pri);
+	{ Reduce(in_op.left_pri);
 	  PushOp();
 	  rmo--;
 
@@ -1453,7 +1551,7 @@ complex_term(const char *stop, term_t term, term_t positions)
 
 	Modify(in_op.left_pri);
 	if ( rmo == 1 )
-	{ Reduce(in_op.op_pri > side_p->right_pri);
+	{ Reduce(in_op.left_pri);
 	  PushOp();	
 	
 	  continue;
@@ -1471,15 +1569,15 @@ complex_term(const char *stop, term_t term, term_t positions)
     if ( rmo != 0 )
       syntaxError("Operator expected");
     rmo++;
+    realloca(out, sizeof(*out), out_n+1);
+    out[out_n].pri = 0;
     out[out_n].term = in;
     out[out_n++].tpos = pin;
-    if	( out_n >= MAX_TERM_NESTING )
-      syntaxError("Operant stack overflow");
   }
 
 exit:
   Modify(OP_MAXPRIORITY+1);
-  Reduce(TRUE);
+  Reduce(OP_MAXPRIORITY+1);
 
   if ( out_n == 1 && side_n == 0 )	/* simple term */
   { PL_assign_term(term, out[0].term);
@@ -1601,9 +1699,18 @@ simple_term(bool must_be_op, term_t term, bool *name, term_t positions)
     case T_PUNCTUATION:
       { switch(token->value.character)
 	{ case '(':
-	    { TRY( complex_term(")", term, positions) );
+	    { int start = token->start;
+
+	      TRY( complex_term(")", term, positions) );
 	      token = get_token(must_be_op);	/* skip ')' */
 
+	      if ( positions )
+	      { Word p = argTermP(*valTermRef(positions), 0);
+
+		*p++ = consInt(start);
+		*p   = consInt(token->end);
+	      }
+
 	      succeed;
 	    }
 	  case '{':
