@@ -1,4 +1,4 @@
-/*  $Id: pl-rec.c,v 1.17 1996/10/15 08:18:47 jan Exp $
+/*  $Id: pl-rec.c,v 1.18 1997/03/10 09:37:04 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -415,7 +415,9 @@ pl_erase(term_t ref)
   if ( !isRecord(record) )
     return warning("erase/1: Invalid reference");
 
+#if O_DEBUGGER
   callEventHook(PLEV_ERASED, record);
+#endif
 
   l = record->list;
   if ( record == l->firstRecord )
