@@ -59,7 +59,7 @@ setupProlog(void)
   initSignals();
 #endif
   if ( status.dumped == FALSE )
-  { hBase = (char *)0xffffffffL;
+  { hBase = (char *)(~0L);
     hTop  = (char *)NULL;
   }
   DEBUG(1, Sdprintf("Stacks ...\n"));
@@ -75,7 +75,7 @@ setupProlog(void)
 
   { void *hbase = allocHeap(sizeof(word));
 
-    base_addresses[STG_HEAP]   = (ulong)hbase & 0xff800000L;
+    base_addresses[STG_HEAP]   = (ulong)hbase & ~0x007fffffL;
     freeHeap(hbase, sizeof(word));
   }
   base_addresses[STG_LOCAL]  = (unsigned long)lBase;
@@ -90,6 +90,11 @@ setupProlog(void)
   DEBUG(1, Sdprintf("base_addresses[STG_TRAIL] = %p\n",
 		    base_addresses[STG_TRAIL]));
 
+#ifdef O_LIMIT_DEPTH
+  depth_limit   = (unsigned long)DEPTH_NO_LIMIT;
+  depth_reached = 0;
+#endif
+
   PL_open_foreign_frame();
 
   if ( status.dumped == FALSE )
@@ -160,7 +165,7 @@ CSetFeature(char *name, char *value)
 }
 
 static void
-CSetIntFeature(char *name, int value)
+CSetIntFeature(char *name, long value)
 { setFeature(lookupAtom(name), consInt(value));
 }
 
@@ -170,6 +175,7 @@ initFeatures()
 #if __WIN32__
   if ( iswin32s() )
     CSetFeature("win32s",	"true");
+  CSetFeature("windows",	"true");
 #endif
   CSetIntFeature("version",	PLVERSION);
   if ( systemDefaults.home )
@@ -229,6 +235,9 @@ initFeatures()
   CSetFeature("float_format", "%g");
   CSetFeature("character_escapes", "true");
   CSetFeature("tty_control", status.notty ? "false" : "true");
+#if defined(__unix__) || defined(unix)
+  CSetFeature("unix", "true");
+#endif
 
 #if defined(__DATE__) && defined(__TIME__)
   { char buf[100];
@@ -423,7 +432,8 @@ extern int errno;
 static int size_alignment;	/* Stack sizes must be aligned to this */
 static int base_alignment;	/* Stack bases must be aligned to this */
 
-#define MB (1024L * 1024L)	/* megabytes */
+#undef MB
+#define MB * (1024L * 1024L)
 
 static long
 align_size(long int x)
@@ -508,7 +518,7 @@ SunOs  4.0.0  on SUN-3 has a bug that causes the various mapped pages to
 point to the same physical memory.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-void
+static void
 mapOrOutOf(Stack s)
 { ulong incr;
 
@@ -532,6 +542,14 @@ mapOrOutOf(Stack s)
   considerGarbageCollect(s);
 }
 
+
+void
+ensureRoomStack(Stack s, int bytes)
+{ while((char *)s->max - (char *)s->top < (int)bytes)
+    mapOrOutOf(s);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 unmap() returns all memory resources of a stack that are  no  longer  in
 use to the OS.
@@ -796,7 +814,7 @@ expandStack(Stack s, caddress addr)
 #include <windows.h>
 #undef small
 
-void
+static void
 mapOrOutOf(Stack s)
 { ulong incr;
 
@@ -822,6 +840,13 @@ mapOrOutOf(Stack s)
 }
 
 
+void
+ensureRoomStack(Stack s, int bytes)
+{ while((char *)s->max - (char *)s->top < (int)bytes)
+    mapOrOutOf(s);
+}
+
+
 static void
 unmap(Stack s)
 { caddress top  = (s->top > s->min ? s->top : s->min);
@@ -835,7 +860,7 @@ unmap(Stack s)
 }
 
 
-#define MAX_VIRTUAL_ALLOC (100 * MB)
+#define MAX_VIRTUAL_ALLOC (100 MB)
 #define SPECIFIC_INIT_STACK 1
 
 static void
@@ -1044,8 +1069,6 @@ of the page size.
 #ifdef FORCED_MALLOC_BASE
 #undef MMAP_MAX_ADDRESS
 #undef MMAP_MIN_ADDRESS
-#undef MB
-#define MB * 1024 * 1024
 #define MMAP_MAX_ADDRESS (FORCED_MALLOC_BASE + 64 MB)
 #define MMAP_MIN_ADDRESS (FORCED_MALLOC_BASE + 16 MB)
 #endif
@@ -1094,6 +1117,8 @@ initStacks(long local, long global, long trail, long argument)
 	   align_base(argument);
   
   large_size = ((space / large) / base_alignment) * base_alignment;
+  if ( large_size > 64 MB )
+    large_size = 64 MB;
   if ( large_size < STACK_MINIMUM )
     fatalError("Can't fit requested stack sizes in address space");
   DEBUG(1, Sdprintf("Large stacks are %ld\n", large_size));
