@@ -1,4 +1,4 @@
-/*  $Id: pl-trace.c,v 1.32 1996/11/11 13:35:44 jan Exp $
+/*  $Id: pl-trace.c,v 1.35 1997/03/10 09:37:11 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -154,7 +154,8 @@ tracePort(LocalFrame frame, LocalFrame bfr, int port, Code PC)
   }
 
   if ( port != BREAK_PORT &&
-       ((!debugstatus.tracing && false(def, SPY_ME))	|| /* non-tracing */
+       ((!debugstatus.tracing &&
+	 (false(def, SPY_ME) || (port & CUT_PORT)))	|| /* non-tracing */
 	debugstatus.skiplevel < levelFrame(frame)	|| /* skipped */
 	((port & (BREAK_PORT|CUT_PORT)) &&
 	 ((debugstatus.skiplevel == levelFrame(frame)) ||
@@ -236,12 +237,12 @@ again:
   Putf("(%3ld) ", levelFrame(frame));
   writeFrameGoal(frame, debugstatus.style);
 
-  debugstatus.skiplevel = VERY_DEEP;
-  debugstatus.tracing = TRUE;
-
   if (debugstatus.leashing & port)
   { char buf[LINESIZ];
 
+    debugstatus.skiplevel = VERY_DEEP;
+    debugstatus.tracing   = TRUE;
+
     Putf(" ? ");
     pl_flush();
     if ( status.notty )
@@ -575,17 +576,14 @@ listProcedure(Definition def)
   qid_t qid;
   extern int Output;
   int OldOut = Output;
-  int debugSave = debugstatus.debugging;
   term_t argv = PL_new_term_refs(1);
   Procedure proc = lookupProcedure(FUNCTOR_listing1, MODULE_system);
 
   unify_definition(argv, def, 0);	/* module:name(args) */
   Output = 1;
-  debugstatus.debugging = FALSE;
   qid = PL_open_query(MODULE_user, FALSE, proc, argv);
   PL_next_solution(qid);
   PL_close_query(qid);
-  debugstatus.debugging = debugSave;
   Output = OldOut;
   PL_discard_foreign_frame(cid);
 }
@@ -673,6 +671,7 @@ traceInterception(LocalFrame frame, LocalFrame bfr, int port, Code PC)
     term_t rarg = argv+3;
     atom_t portname = NULL_ATOM;
     functor_t portfunc = NULL;
+    int nodebug = FALSE;
 
     switch(port)
     { case CALL_PORT:	  portname = ATOM_call;		break;
@@ -707,7 +706,6 @@ traceInterception(LocalFrame frame, LocalFrame bfr, int port, Code PC)
     PL_put_integer(argv+2, PrologRef(bfr));
     PL_put_variable(rarg);
 
-    debugstatus.suspendTrace++;
     qid = PL_open_query(MODULE_user, FALSE, proc, argv);
     if ( PL_next_solution(qid) )
     { atom_t a;
@@ -715,7 +713,10 @@ traceInterception(LocalFrame frame, LocalFrame bfr, int port, Code PC)
       if ( PL_get_atom(rarg, &a) )
       { if ( a == ATOM_continue )
 	  rval = ACTION_CONTINUE;
-	else if ( a == ATOM_fail )
+	else if ( a == ATOM_nodebug )
+	{ rval = ACTION_CONTINUE;
+	  nodebug = TRUE;
+	} else if ( a == ATOM_fail )
 	  rval = ACTION_FAIL;
 	else if ( a == ATOM_retry )
 	  rval = ACTION_RETRY;
@@ -734,7 +735,10 @@ traceInterception(LocalFrame frame, LocalFrame bfr, int port, Code PC)
     }
     PL_close_query(qid);
     PL_discard_foreign_frame(cid);
-    debugstatus.suspendTrace--;
+
+    if ( nodebug )
+      pl_nodebug();
+
   }
 
   return rval;
@@ -899,6 +903,8 @@ initTracer(void)
 		*       PROLOG PREDICATES       *
 		*********************************/
 
+#if O_DEBUGGER
+
 int
 tracemode(int doit, int *old)
 { if ( doit )
@@ -939,6 +945,19 @@ debugmode(int doit, int *old)
   succeed;
 }
 
+#else /*O_DEBUGGER*/
+
+int
+tracemode(int doit, int *old)
+{ succeed;
+}
+
+int
+debugmode(int doit, int *old)
+{ succeed;
+}
+
+#endif
 
 word
 pl_trace()
@@ -962,7 +981,10 @@ pl_debug()
 
 word
 pl_nodebug()
-{ return debugmode(FALSE, NULL);
+{ tracemode(FALSE, NULL);
+  debugmode(FALSE, NULL);
+
+  succeed;
 }
 
 word
@@ -973,6 +995,7 @@ pl_debugging()
 word
 pl_skip_level(term_t old, term_t new)
 { atom_t a;
+  long sl;
 
   if ( debugstatus.skiplevel == VERY_DEEP )
   { TRY(PL_unify_atom(old, ATOM_very_deep));
@@ -980,8 +1003,10 @@ pl_skip_level(term_t old, term_t new)
   { TRY(PL_unify_integer(old, debugstatus.skiplevel));
   }
       
-  if ( PL_get_long(new, &debugstatus.skiplevel) )
+  if ( PL_get_long(new, &sl) )
+  { debugstatus.skiplevel = (unsigned long) sl;
     succeed;
+  }
   if ( PL_get_atom(new, &a) && a == ATOM_very_deep)
   { debugstatus.skiplevel = VERY_DEEP;
     succeed;
@@ -1031,6 +1056,7 @@ pl_debuglevel(term_t old, term_t new)
 }
 
 
+
 word
 pl_unknown(term_t old, term_t new)
 { Module m = contextModule(environment_frame);
@@ -1081,6 +1107,8 @@ pl_prolog_frame_attribute(term_t frame, term_t what,
   if ((fr = FrameRef(fri)) < lBase || fr > lTop)
     return warning("prolog_frame_attribute/3: illegal frame reference");
 
+  set(fr, FR_WATCHED);			/* explicit call to do this? */
+
   if ( key == ATOM_argument && arity == 1 )
   { term_t arg = PL_new_term_ref();
     int argn;
@@ -1192,8 +1220,10 @@ pl_prolog_frame_attribute(term_t frame, term_t what,
 }
 
 
+#if O_DEBUGGER
+
 		 /*******************************
-		 *	  PROLOG EVENET HOOK	*
+		 *	  PROLOG EVENT HOOK	*
 		 *******************************/
 
 void
@@ -1241,6 +1271,13 @@ callEventHook(int ev, ...)
 					             : ATOM_false);
 	break;
       }
+      case PLEV_FRAMEFINISHED:
+      { LocalFrame fr = va_arg(args, LocalFrame);
+
+	PL_unify_term(arg, PL_FUNCTOR, FUNCTOR_frame_finished1,
+		           PL_INTEGER, PrologRef(fr));
+	break;
+      }
       default:
 	warning("callEventHook(): unknown event: %d", ev);
         goto out;
@@ -1252,3 +1289,5 @@ callEventHook(int ev, ...)
     va_end(args);
   }
 }
+
+#endif /*O_DEBUGGER*/
