@@ -1,4 +1,4 @@
-/*  $Id: pl-write.c,v 1.16 1996/10/02 16:33:26 jan Exp $
+/*  $Id: pl-write.c,v 1.18 1997/03/10 09:37:19 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -51,7 +51,7 @@ atomType(atom_t a)
     return *s2 == EOS ? AT_LOWER : AT_QUOTE;
   }
 
-  if ( streq(s, ".") )
+  if ( a == ATOM_dot )
     return AT_FULLSTOP;
   
   if (isSymbol(*s))
@@ -62,10 +62,11 @@ atomType(atom_t a)
     return *s2 == EOS ? AT_SYMBOL : AT_QUOTE;
   }
 
-  if ((isSolo(*s) || *s == ',') && s[1] == EOS)
+					/* % should be quoted! */
+  if ((isSolo(*s) || *s == ',') && s[1] == EOS && s[0] != '%' )
     return AT_SOLO;
 
-  if (streq(s, "[]") || streq(s, "{}") )
+  if ( a == ATOM_nil || a == ATOM_curl )
     return AT_SPECIAL;
   
   return AT_QUOTE;
@@ -298,11 +299,9 @@ pl_call2(term_t goal, term_t arg)
   }
   PL_put_term(argv+n, arg);
 
-  debugstatus.suspendTrace++;
   qid = PL_open_query(mod, FALSE, proc, argv);
   rval = PL_next_solution(qid);
   PL_close_query(qid);
-  debugstatus.suspendTrace--;
 
 out:
   PL_discard_foreign_frame(cid);
